@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
  *
  *			    http://www.ntop.org/
  *
@@ -9,7 +9,6 @@
  *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
@@ -698,6 +697,99 @@ void doAddURL(int len) {
   }
 }
 
+/* ********************************* */
+
+void setPcapFilter(char* filters, int device_id) {
+  char knownFilters[2048];
+
+  knownFilters[0] = '\0';
+
+  if(filters != NULL) {
+    char *filters_copy;
+    char buf[2048];
+
+    if(filters[0] == '@') {
+      /* This is a file */
+      filters_copy = read_file(filters, buf, sizeof(buf));
+      if(filters_copy)
+	filters_copy = strdup(buf);
+    } else
+      filters_copy = strdup(filters);
+
+    if(filters_copy) {
+      char find_token[32], *the_filter;
+
+      /* 
+	 Format is
+	 eth0="filter for eth0",eth1="filter for eth1"....
+	 or
+	 "filter for all interfaces"
+      */
+
+      if(strstr(filters_copy, "=") == NULL) {
+	/* This is a filter for all interfaces */
+	the_filter = filters_copy;
+
+      } else {
+	safe_snprintf(__FILE__, __LINE__,
+		      find_token, sizeof(find_token), 
+		      "%s=", myGlobals.device[device_id].name);
+		
+	the_filter = strstr(filters_copy, find_token);
+	if(the_filter != NULL) {
+	  /* We have an interface specific filter */
+	  char *quote;
+	  
+	  the_filter += strlen(find_token);	  
+
+	  quote = strchr(the_filter, ',');
+	  if(quote != NULL) quote[0] = '\0';	  
+	} else {
+	  traceEvent(CONST_TRACE_INFO, 
+		     "No filter specified for interface %s", 
+		     myGlobals.device[device_id].name);
+	  return;
+	}
+      }
+
+      if(the_filter != NULL) {
+	struct bpf_program fcode;
+
+	traceEvent(CONST_TRACE_INFO, 
+		   "Using filter '%s' for interface %s", 
+		   the_filter, myGlobals.device[device_id].name);
+
+	if(myGlobals.device[device_id].pcapPtr
+	   && (!myGlobals.device[device_id].virtualDevice)) {
+	  if((pcap_compile(myGlobals.device[device_id].pcapPtr, &fcode,
+			   the_filter, 1,
+			   myGlobals.device[device_id].netmask.s_addr) < 0)
+	     || (pcap_setfilter(myGlobals.device[device_id].pcapPtr, &fcode) < 0)) {
+	    traceEvent(CONST_TRACE_ERROR,
+		       "Wrong filter '%s' (%s) on interface %s",
+		       the_filter,
+		       pcap_geterr(myGlobals.device[device_id].pcapPtr),
+		       myGlobals.device[device_id].name);
+	  } else {
+	    pcap_freecode(&fcode);
+
+	    if(*the_filter!='\0') {
+	      traceEvent(CONST_TRACE_ALWAYSDISPLAY, "Set filter \"%s\" on interface %s",
+			 the_filter, myGlobals.device[device_id].name);
+	    } else {
+	      traceEvent(CONST_TRACE_ALWAYSDISPLAY, 
+			 "Set no kernel (libpcap) filtering on interface %s",
+			 myGlobals.device[device_id].name);
+	    }
+	  }
+	}       
+      }
+      
+      free(filters_copy);
+    }
+  }
+}
+
 /* *******************************/
 
 /* Courtesy of Michael Weidel <michael.weidel@gmx.de> */
@@ -725,6 +817,7 @@ int doChangeFilter(int len) {
       key = NULL;
     }
   }
+
   if(key == NULL) {
     decodeWebFormURL(myGlobals.runningPref.currentFilterExpression);
     for(i=0; i<strlen(myGlobals.runningPref.currentFilterExpression); i++) {
@@ -735,39 +828,17 @@ int doChangeFilter(int len) {
        break;
       }
     }
-  } else err = "ERROR: The HTTP Post Data was invalid.";
+  } else 
+    err = "ERROR: The HTTP Post Data was invalid.";
+
   if(badChar)
     err = "ERROR: the specified filter expression contains invalid characters.";
 
-  if(err==NULL) {
+  if(err == NULL) {
     traceEvent(CONST_TRACE_INFO, "Changing the kernel (libpcap) filter...");
 
-    for(i=0; i<myGlobals.numDevices; i++) {
-      if(myGlobals.device[i].pcapPtr && (!myGlobals.device[i].virtualDevice) && (err==NULL)) {
-	if((pcap_compile(myGlobals.device[i].pcapPtr, &fcode, myGlobals.runningPref.currentFilterExpression, 1,
-			myGlobals.device[i].netmask.s_addr) < 0)
-	   || (pcap_setfilter(myGlobals.device[i].pcapPtr, &fcode) < 0)) {
-	  traceEvent(CONST_TRACE_ERROR,
-		     "Wrong filter '%s' (%s) on interface %s - using old filter",
-		     myGlobals.runningPref.currentFilterExpression,
-		     pcap_geterr(myGlobals.device[i].pcapPtr), myGlobals.device[i].name);
-
-	  err="The syntax of the defined filter is wrong.";
-
-	} else {
-
-         pcap_freecode(&fcode);
-
-	 if(*myGlobals.runningPref.currentFilterExpression!='\0'){
-	   traceEvent(CONST_TRACE_ALWAYSDISPLAY, "Set filter \"%s\" on interface %s",
-		      myGlobals.runningPref.currentFilterExpression, myGlobals.device[i].name);
-	 } else {
-	   traceEvent(CONST_TRACE_ALWAYSDISPLAY, "Set no kernel (libpcap) filtering on interface %s",
-		      myGlobals.device[i].name);
-	 }
-	}
-      }
-    }
+    for(i=0; i<myGlobals.numDevices; i++)
+      setPcapFilter(myGlobals.runningPref.currentFilterExpression, i);
   }
 
   sendHTTPHeader(FLAG_HTTP_TYPE_HTML, 0, 1);
@@ -798,25 +869,10 @@ int doChangeFilter(int len) {
   if(myGlobals.runningPref.currentFilterExpression != NULL)
     free(myGlobals.runningPref.currentFilterExpression);
 
+  /* restore old filter expression */
   myGlobals.runningPref.currentFilterExpression = currentFilterExpressionSav;
-  for(i=0; i<myGlobals.numDevices; i++) {      /* restore old filter expression */
-    if(myGlobals.device[i].pcapPtr && (!myGlobals.device[i].virtualDevice) && (err==NULL)) {
-      if((pcap_compile(myGlobals.device[i].pcapPtr, &fcode,
-                       myGlobals.runningPref.currentFilterExpression, 1,
-                       myGlobals.device[i].netmask.s_addr) < 0)) {
-        if((pcap_setfilter(myGlobals.device[i].pcapPtr, &fcode) < 0)) {
-          traceEvent(CONST_TRACE_ERROR,
-                     "Wrong filter '%s' (%s) on interface %s - using old filter",
-                     myGlobals.runningPref.currentFilterExpression,
-                     pcap_geterr(myGlobals.device[i].pcapPtr), myGlobals.device[i].name);
-
-        }
-
-        pcap_freecode(&fcode);
-      }
-    }
-
-  }
+  for(i=0; i<myGlobals.numDevices; i++)
+    setPcapFilter(myGlobals.runningPref.currentFilterExpression, i); 
 
   printFlagedWarning(err);
   printHTMLtrailer();
@@ -1131,12 +1187,17 @@ void addDefaultAdminUser(void) {
    tmpPrefs.devices = NULL;
 
    while(token != NULL) {
-     char *key, *value;
+     char *key, *value, value_buf[2048];
 
      key = strtok_r(token, "=", &strtokState);
      if(key != NULL) value = strtok_r(NULL, "=", &strtokState); else value = NULL;
 
-     /* traceEvent(CONST_TRACE_INFO, "RRD: key(%s)=%s", key, value); */
+	 if(value != NULL) {
+	  safe_snprintf (__FILE__, __LINE__, value_buf, sizeof(value_buf),
+					 "%s", value);
+  	  unescape_url(value_buf);
+      value = value_buf;
+	 }
 
      if(key) {
        action = processNtopPref(key, value, savePref, &tmpPrefs);
@@ -1461,7 +1522,7 @@ char * rindex(const char *p, int ch) {
 
 void handleNtopConfig(char* url, UserPrefDisplayPage configScr,
 		      int postLen) {
-  char buf[1024], hostStr[MAXHOSTNAMELEN+16];
+  char buf[4096], token_buf[512], hostStr[MAXHOSTNAMELEN+16];
   bool startCap = FALSE;
   int len;
   UserPref defaults, *pref = &myGlobals.savedPref;
@@ -1488,13 +1549,17 @@ void handleNtopConfig(char* url, UserPrefDisplayPage configScr,
        */
 
       if((buf != NULL) && (buf [0] != '\0')) {
-	unescape_url(buf);
 
 	/* traceEvent (CONST_TRACE_INFO, "BUF='%s'\n", buf); */
 
 	/* locate the last parameter which tells us which button got pressed */
 	if ((token = rindex (buf, '&')) != NULL) {
 	  token++;
+      safe_snprintf(__FILE__, __LINE__, token_buf, sizeof(token_buf),
+					"%s", token);
+  	  unescape_url(token_buf);
+	  token = token_buf;
+
 	  if (strncmp (token, NTOP_SAVE_PREFS,
 		       strlen (NTOP_SAVE_PREFS)) == 0) {
 	    savePref = TRUE;
@@ -1746,7 +1811,7 @@ void handleNtopConfig(char* url, UserPrefDisplayPage configScr,
 		     "Limit number of IP sessions entries created in order"
 		     " to limit memory used by ntop");
 
-    CONFIG_RADIO_ENTRY(DARK_BG, "Don't Merge Interfaces (-M)",
+    CONFIG_RADIO_ENTRY(DARK_BG, "Merge Interfaces (-M)",
 		       NTOP_PREF_MERGEIF, pref->mergeInterfaces,
 		       "Yes = merge data from all interfaces (if possible), No = do not merge data from all interfaces");
 
