@@ -5,7 +5,7 @@
 # -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 #
 # Copyright (C) 2004-06 Rocco Carbone <rocco@ntop.org>
-#               2006-07 Luca Deri <deri@ntop.org>
+#               2006-09 Luca Deri <deri@ntop.org>
 #
 # -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 #
@@ -244,6 +244,16 @@ AUTOCONF=`find_command 'autoconf*'`
   GNU_OR_DIE=0
 }
 
+WGET=`find_command 'wget*'`
+($WGET --version) < /dev/null > /dev/null 2>&1 ||
+{
+  echo
+  echo "You must have wget installed to compile $progname."
+  echo "Download the appropriate package for your distribution, or get the"
+  echo "source tarball from ftp://ftp.gnu.org/pub/gnu/wget"
+  GNU_OR_DIE=0
+}
+
 if test "$GNU_OR_DIE" -eq 0; then
   exit 1
 fi
