@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2001-04 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 2001-09 Luca Deri <deri@ntop.org>
  *
  *     		            http://www.ntop.org/
  *
@@ -41,7 +41,7 @@ static void sendEmitterString(FILE *fDescr, char *theString) {
   if(fDescr == NULL)
     sendString(theString);
   else
-    fprintf(fDescr, theString);
+    fprintf(fDescr, "%s", theString);
 }
 
 /* *************************** */
