@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- * Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
+ *          Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -100,17 +100,18 @@ void initGdbm(char *prefDirectory,  /* Directory with persistent files */
     return;
   }
 
-  initSingleGdbm(&myGlobals.addressQueueFile, "addressQueue.db", spoolDirectory, TRUE,  NULL);
   initSingleGdbm(&myGlobals.dnsCacheFile,     "dnsCache.db",     spoolDirectory, -1,    NULL);
-  initSingleGdbm(&myGlobals.macPrefixFile,    "macPrefix.db",    spoolDirectory, FALSE,  &statbuf);
-  initSingleGdbm(&myGlobals.fingerprintFile,  "fingerprint.db",  spoolDirectory, FALSE,  &statbuf);
-  createVendorTable(&statbuf);
+
+  if(!myGlobals.runningPref.liveMode) {
+    initSingleGdbm(&myGlobals.macPrefixFile,    "macPrefix.db",    spoolDirectory, FALSE,  &statbuf);
+    initSingleGdbm(&myGlobals.fingerprintFile,  "fingerprint.db",  spoolDirectory, FALSE,  &statbuf);
+    createVendorTable(&statbuf);
+  }
 }
 
 /* ******************************* */
 
 static void allocateOtherHosts() {
-
   if(myGlobals.otherHostEntry != NULL) {
     traceEvent(CONST_TRACE_WARNING, "Attempting to call twice allocateOtherHosts()");
     return;
@@ -174,7 +175,7 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
     traceEvent(CONST_TRACE_ALWAYSDISPLAY, "NOTE: Interface merge enabled by default");
 
   myGlobals.checkVersionStatus = FLAG_CHECKVERSION_NOTCHECKED;
-  myGlobals.checkVersionStatusAgain = 0;
+  myGlobals.checkVersionStatusAgain = 1;
 
 #if !defined(WIN32) && defined(HAVE_LIBWRAP)
   allow_severity = DEFAULT_TCPWRAP_ALLOW;
@@ -210,11 +211,9 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
     exit(-1);
   }
 
-
   /* Databases */
   myGlobals.dnsCacheFile = NULL;
   myGlobals.pwFile = NULL;
-  myGlobals.addressQueueFile = NULL;
 
   /* the table of broadcast entries */
   myGlobals.broadcastEntry = NULL;
@@ -227,9 +226,6 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
 
   myGlobals.numThreads = 0;            /* # of running threads */
 
-  for (i = 0; i < MAX_NUM_DEQUEUE_ADDRESS_THREADS; i ++)
-    myGlobals.dequeueAddressThreadId[i] = (pthread_t)-1;
-
 #ifdef HAVE_OPENSSL
   myGlobals.sslInitialized = 0;
   myGlobals.runningPref.sslPort = 0; /* Disabled by default: enabled via -W */
@@ -241,35 +237,6 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
   myGlobals.dnsSniffARPACount = 0;
   myGlobals.dnsSniffStoredInCache = 0;
 
-  myGlobals.addressQueuedCount = 0;
-  myGlobals.addressQueuedDup = 0;
-  myGlobals.addressQueuedCurrent = 0;
-  myGlobals.addressQueuedMax = 0;
-
-  /* Address Resolution counters */
-  myGlobals.numipaddr2strCalls = 0;
-  myGlobals.numResolveAddressCalls = 0;
-  myGlobals.numResolveNoCacheDB = 0;
-  myGlobals.numResolveCacheDBLookups = 0;
-  myGlobals.numResolvedFromCache = 0;
-#ifdef PARM_USE_HOST
-  myGlobals.numResolvedFromHostAddresses = 0;
-#endif
-  myGlobals.numAttemptingResolutionWithDNS = 0;
-  myGlobals.numResolvedWithDNSAddresses = 0;
-  myGlobals.numDNSErrorHostNotFound = 0;
-  myGlobals.numDNSErrorNoData = 0;
-  myGlobals.numDNSErrorNoRecovery = 0;
-  myGlobals.numDNSErrorTryAgain = 0;
-  myGlobals.numDNSErrorOther = 0;
-  myGlobals.numKeptNumericAddresses = 0;
-  myGlobals.dnsCacheStoredLookup = 0;
-
-  myGlobals.numFetchAddressFromCacheCalls = 0;
-  myGlobals.numFetchAddressFromCacheCallsOK = 0;
-  myGlobals.numFetchAddressFromCacheCallsFAIL = 0;
-  myGlobals.numFetchAddressFromCacheCallsSTALE = 0;
-
   /* Misc */
   myGlobals.separator = "&nbsp;";
 
@@ -334,13 +301,8 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
   /* traceEvent(CONST_TRACE_INFO, "NOTE: atfork() handler registered for mutexes, rc %d", i); */
 #endif
 
-  /*
-   * Create two variables (semaphores) used by functions in pbuf.c to queue packets
-   */
-  createCondvar(&myGlobals.queueAddressCondvar);
-  createMutex(&myGlobals.queueAddressMutex);
-
   createMutex(&myGlobals.gdbmMutex);        /* data to synchronize thread access to db files */
+  createMutex(&myGlobals.portsMutex);       /* Avoid race conditions while handling ports */
 
   for(i=0; i<NUM_SESSION_MUTEXES; i++)
     createMutex(&myGlobals.tcpSessionsMutex[i]); /* data to synchronize TCP sessions access */
@@ -527,12 +489,65 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
   myGlobals.fcMatrixHashCollisions = 0;
   myGlobals.fcMatrixHashUnresCollisions = 0;
 
-  myGlobals.calculateEfficiency = 1;
+  /* 
+     Efficiency is the ability to fill-up ATM cells so that they
+     can be efficiently used by applications
+  */
   myGlobals.cellLength = 47; /* FIX - this is valid only for ATM */
 }
 
 /* ********************************* */
 
+static void loadGeoIP() {
+  int i;
+  struct stat statbuf;
+
+  /* Initialize GeoIP databases */
+  for(i=0; myGlobals.configFileDirs[i] != NULL; i++) {
+    char path[256];
+    
+    safe_snprintf(__FILE__, __LINE__, path, sizeof(path),
+		  "%s%c%s",
+		  myGlobals.configFileDirs[i], 
+		  CONST_PATH_SEP, GEO_IP_FILE);
+    revertSlashIfWIN32(path, 0);
+
+    if(stat(path, &statbuf) == 0) {
+      if((myGlobals.geo_ip_db = GeoIP_open(path, GEOIP_CHECK_CACHE)) != NULL) {
+	traceEvent(CONST_TRACE_INFO, "GeoIP: loaded config file %s", path);
+	break;
+      }
+    }
+  }
+  
+  if(myGlobals.geo_ip_db == NULL)
+    traceEvent(CONST_TRACE_ERROR, "GeoIP: unable to load file %s", GEO_IP_FILE);
+  
+  /* *************************** */
+
+  for(i=0; myGlobals.configFileDirs[i] != NULL; i++) {
+    char path[256];
+    
+    safe_snprintf(__FILE__, __LINE__, path, sizeof(path),
+		  "%s%c%s",
+		  myGlobals.configFileDirs[i], 
+		  CONST_PATH_SEP, GEO_IP_ASN_FILE);
+    revertSlashIfWIN32(path, 0);
+
+    if(stat(path, &statbuf) == 0) {
+      if((myGlobals.geo_ip_asn_db = GeoIP_open(path, GEOIP_CHECK_CACHE)) != NULL) {
+	traceEvent(CONST_TRACE_INFO, "GeoIP: loaded ASN config file %s", path);
+	break;
+      }
+    }
+  }
+  
+  if(myGlobals.geo_ip_asn_db == NULL)
+    traceEvent(CONST_TRACE_ERROR, "GeoIP: unable to load ASN file %s", GEO_IP_ASN_FILE);  
+}
+
+/* ********************************* */
+
 void initNtop(char *devices) {
   char value[32];
 
@@ -590,7 +605,6 @@ void initNtop(char *devices) {
       }
     }
 
-
     if(!found) {
       traceEvent(CONST_TRACE_WARNING, "ntop will not become a daemon as it has not been");
       traceEvent(CONST_TRACE_WARNING, "installed properly (did you do 'make install')");
@@ -642,6 +656,9 @@ void initNtop(char *devices) {
   addNewIpProtocolToHandle("OSPF", 89, 0 /* no proto */);
   addNewIpProtocolToHandle("IPsec", 50, 51);
 
+  init_maps();
+  loadGeoIP();
+
   if(fetchPrefsValue("globals.displayPolicy", value, sizeof(value)) == -1) {
     myGlobals.hostsDisplayPolicy = showAllHosts /* 0 */;
     storePrefsValue("globals.displayPolicy", "0");
@@ -666,7 +683,6 @@ void initNtop(char *devices) {
       myGlobals.localityDisplayPolicy = showSentReceived;
   }
 
-
   if(myGlobals.runningPref.skipVersionCheck != TRUE) {
     pthread_t myThreadId;
     createThread(&myThreadId, checkVersion, NULL);
