@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- * Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
+ *            Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -176,6 +176,7 @@ extern void changeFilter(void);
 extern void setAdminPassword(char* pass);
 extern void addDefaultAdminUser(void);
 extern void printNtopConfigHeader(char *url, UserPrefDisplayPage configScr);
+extern void setPcapFilter(char* filters, int device_id);
 
 /* dataFormat.c */
 extern char* formatKBytes(float numKBytes, char *outStr, int outStrLen);
@@ -727,7 +728,6 @@ extern u_int numActiveNxPorts(u_int deviceId);
 extern u_int numActiveVsans(u_int deviceId);
 extern u_int32_t xaton(char *s);
 extern void* addNodeInternal(u_int32_t ip, int prefix, char *country, int as);
-extern char *ip2CountryCode(HostAddr ip);
 extern short addrcmp(HostAddr *addr1, HostAddr *addr2);
 extern HostAddr     * addrcpy(HostAddr *dst, HostAddr *src);
 extern int            addrinit(HostAddr *addr);
@@ -742,7 +742,6 @@ extern unsigned short computeIdx(HostAddr *srcAddr, HostAddr *dstAddr,
 extern u_int16_t computeTransId(HostAddr *srcAddr, HostAddr *dstAddr,
 				int sport, int dport);
 extern int setSpecifiedUser(void);
-extern u_short ip2AS(HostAddr ip);
 extern u_int16_t getHostAS(HostTraffic *el);
 extern int emptySerial(HostSerial *a);
 extern int cmpSerial(HostSerial *a, HostSerial *b);
@@ -807,7 +806,7 @@ extern void freePortsUsage(HostTraffic *el);
 extern PortUsage* getPortsUsage(HostTraffic *el, u_int portIdx, int createIfNecessary);
 extern char* vlan2name(u_int16_t vlanId, char *buf, int buf_len);
 extern void mkdir_p(char *tag, char *path, int permission);
-
+extern char* read_file(char* path, char* buf, u_int buf_len);
 /* Formatting for %.2f ... */
 #define xvertDOT00MB(v)(((float)(v)/(float)(1024.0*1024.0))+0.005)
 #define xvertDOT00KB(v) (((float)(v)/(float)(1024.0))+0.005)
@@ -893,9 +892,6 @@ extern int insert_flow_record(u_int16_t probeId,
 			      u_int32_t first, u_int32_t last,
 			      u_int16_t srcPort, u_int16_t dstPort, u_int8_t tcpFlags,
 			      u_int8_t proto, u_int8_t tos, u_int16_t vlanId);
-#ifdef HAVE_NETDB_H
-extern int h_errno; /* netdb.h */
-#endif
 
 /* Pseudo-functions.
  *   We use these as if they were real functions, but they expand to
