@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
  *
  *		 	    http://www.ntop.org/
  *
@@ -361,4 +361,12 @@ extern void drawVsanStatsPktsDistribution(int deviceId);
 extern void drawVsanDomainTrafficDistribution(u_short vsanId, u_char dataSent);
 extern void drawVsanSwilsProtoDistribution(u_short vsanId);
 
+/* perl.c */
 extern int handlePerlHTTPRequest(char *url);
+
+/* lua.c */
+extern int handleLuaHTTPRequest(char *url);
+
+/* map.c */
+extern void init_maps(void);
+extern void create_host_map(void);
