@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- * Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
+ *         Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -835,6 +835,7 @@ typedef struct hostTraffic {
   u_int16_t        vlanId;          /* VLAN Id (-1 if not set) */
   u_int16_t        ifId;            /* Interface Id [e.g. for NetFlow] (-1 if not set) */
   u_int16_t        hostAS;          /* AS to which the host belongs to */
+  char             *hostASDescr;    /* Description of the host AS */
   time_t           firstSeen, lastSeen; /* time when this host has sent/rcvd some data  */
   u_char           ethAddress[LEN_ETHERNET_ADDRESS];
   u_char           lastEthAddress[LEN_ETHERNET_ADDRESS]; /* used for remote addresses */
@@ -842,11 +843,12 @@ typedef struct hostTraffic {
   char             hostNumIpAddress[20] /* xxx.xxx.xxx.xxx */, *dnsDomainValue, *dnsTLDValue;
   u_int8_t         network_mask;    /* IPv6 notation e.g. /24 */
   int8_t           known_subnet_id; /* UNKNOWN_SUBNET_ID if the host does not belong to a known subnet */
-  char             *hwModel, *description, *community, *ip2ccValue, *fingerprint;
+  char             *hwModel, *description, *community, *fingerprint;
   char             hostResolvedName[MAX_LEN_SYM_HOST_NAME];
   short            hostResolvedNameType;
   u_short          minTTL, maxTTL; /* IP TTL (Time-To-Live) */
   struct timeval   minLatency, maxLatency;
+  GeoIPRecord      *geo_ip;
 
   TrafficCounter   greSent, greRcvd, greEfficiencySent, greEfficiencyRcvd,
     grePktSent, grePktRcvd, lastGrePktSent, lastGrePktRcvd;
@@ -893,10 +895,8 @@ typedef struct hostTraffic {
   TrafficCounter   tcpSentLoc, tcpSentRem, udpSentLoc, udpSentRem, icmpSent,icmp6Sent;
   TrafficCounter   tcpRcvdLoc, tcpRcvdFromRem, udpRcvdLoc, udpRcvdFromRem, icmpRcvd, icmp6Rcvd;
 
-  TrafficCounter   tcpFragmentsSent,  tcpFragmentsRcvd,
-    udpFragmentsSent, udpFragmentsRcvd,
-    icmpFragmentsSent, icmpFragmentsRcvd,
-    icmp6FragmentsSent, icmp6FragmentsRcvd;
+  TrafficCounter   tcpFragmentsSent,  tcpFragmentsRcvd, udpFragmentsSent, udpFragmentsRcvd,
+    icmpFragmentsSent, icmpFragmentsRcvd, icmp6FragmentsSent, icmp6FragmentsRcvd;
 
   /* Protocol decoders */
   ProtocolInfo     *protocolInfo;
@@ -1999,15 +1999,6 @@ struct option
 };
 #endif /* HAVE_GETOPT_H */
 
-/* Courtesy of Andreas Pfaller <apfaller@yahoo.com.au> */
-typedef struct IPNode {
-  struct IPNode *b[2];
-  union {
-    char cc[4]; /* Country */
-    u_short as; /* AS */
-  } node;
-} IPNode;
-
 /* *************************************************************** */
 
 typedef enum {
@@ -2072,82 +2063,84 @@ typedef enum {
  */
 
 typedef struct _userPref {
-  char *accessLogFile;          /* -a |--access-log-file */
-  bool enablePacketDecoding;    /* -b | --disable-decoders */
-  bool stickyHosts;             /* -c | --sticky-hosts */
-  bool daemonMode;              /* -d | --daemon */
-  int  maxNumLines;             /* -e | --max-table-rows */
-  bool trackOnlyLocalHosts;     /* -g | --track-local-hosts */
-  char *devices;                /* -i | --interface */
-  bool enableOtherPacketDump;   /* -j | --create-other-packets */
-  char *pcapLog;                /* -l | --pcap-log */
-  char *localAddresses;         /* -m | --local-subnets */
-  bool numericFlag;             /* -n | --numeric-ip-addresses */
-  bool dontTrustMACaddr;        /* -o | --no-mac */
-  char *protoSpecs;             /* -p | --protocols */
-  bool enableSuspiciousPacketDump; /* -q | --create-suspicious-packets */
-  int  refreshRate;             /* -r | --refresh-time */
-  bool disablePromiscuousMode;  /* -s | --no-promiscuous */
-  int  traceLevel;              /* -t | --trace-level */
-  char *mapperURL;              /* -U | --disable-mapper */
-  u_int     maxNumHashEntries;  /* -x */
-  u_int     maxNumSessions;     /* -X */
-
-  /* SQL Database */
+  char *accessLogFile;           /* -a |--access-log-file */
+  bool enablePacketDecoding;     /* -b | --disable-decoders */
+  bool stickyHosts;              /* -c | --sticky-hosts */
+  bool daemonMode;               /* -d | --daemon */
+  int  maxNumLines;              /* -e | --max-table-rows */
+  bool trackOnlyLocalHosts;      /* -g | --track-local-hosts */
+  char *devices;                 /* -i | --interface */
+  bool enableOtherPacketDump;    /* -j | --create-other-packets */
+  char *pcapLog;                 /* -l | --pcap-log */
+  char *localAddresses;          /* -m | --local-subnets */
+  bool numericFlag;              /* -n | --numeric-ip-addresses */
+  bool dontTrustMACaddr;         /* -o | --no-mac */
+  char *protoSpecs;              /* -p | --protocols */
+  bool enableSuspiciousPacketDump;  /* -q | --create-suspicious-packets */
+  int  refreshRate;              /* -r | --refresh-time */
+  bool disablePromiscuousMode;   /* -s | --no-promiscuous */
+  int  traceLevel;               /* -t | --trace-level */
+  char *mapperURL;               /* -U | --disable-mapper */
+  u_int     maxNumHashEntries;   /* -x */
+  u_int     maxNumSessions;      /* -X */
+  u_int8_t  liveMode;            /* --live */
+  u_int8_t  calculateEfficiency; /* --enable-efficiency */
+  
+   /* SQL Database */
   char    sqlDbConfig[64];
   u_int   sqlRecDaysLifetime;
   bool    saveRecordsIntoDb, saveSessionsIntoDb;
 
-  u_int16_t defaultVsan;        /* -v | --default-vsan */
-  char      *webAddr;           /* -w | --http-serveraddress[:port] */
+  u_int16_t defaultVsan;         /* -v | --default-vsan */
+  char      *webAddr;            /* -w | --http-serveraddress[:port] */
   int       webPort;
-  int       ipv4or6;            /* -6 -4 */
-  bool      enableSessionHandling; /* -z | --disable-sessions */
+  int       ipv4or6;             /* -6 -4 */
+  bool      enableSessionHandling;  /* -z | --disable-sessions */
 
-  char *currentFilterExpression;/* -B | --filter-expression */
-  u_short samplingRate;         /* -C | --sampling-rate */
-  char domainName[MAXHOSTNAMELEN]; /* -D | --domain */
-  char *flowSpecs;              /* -F | --flow-spec */
+  char *currentFilterExpression; /* -B | --filter-expression */
+  u_short samplingRate;          /* -C | --sampling-rate */
+  char domainName[MAXHOSTNAMELEN];  /* -D | --domain */
+  char *flowSpecs;               /* -F | --flow-spec */
 
-  bool debugMode;               /* -K | --enable-debug */
+  bool debugMode;                /* -K | --enable-debug */
 #ifndef WIN32
-  int  useSyslog;               /* -L | --use-syslog*/
+  int  useSyslog;                /* -L | --use-syslog*/
 #endif
 
-  bool mergeInterfaces;         /* -M | --no-interface-merge */
-  bool enableL7;                /* Enable/disable l7 protocol pattern matching */
-  char *pcapLogBasePath;        /* -O | --pcap-file-path */ /* Added by Ola Lundqvist <opal@debian.org>. */
-  char *fcNSCacheFile;          /* -N | --wwn-map */
+  bool mergeInterfaces;          /* -M | --no-interface-merge */
+  bool enableL7;                 /* Enable/disable l7 protocol pattern matching */
+  char *pcapLogBasePath;         /* -O | --pcap-file-path */  /* Added by Ola Lundqvist <opal@debian.org>. */
+  char *fcNSCacheFile;           /* -N | --wwn-map */
 
 #ifdef HAVE_OPENSSL
-  char *sslAddr;                /* -W | --https-serveraddress[:port] */
+  char *sslAddr;                 /* -W | --https-serveraddress[:port] */
   int  sslPort;
 #endif
 
 #ifdef MAKE_WITH_SSLWATCHDOG_RUNTIME
-  bool useSSLwatchdog;          /* --ssl-watchdog '133' */
+  bool useSSLwatchdog;           /* --ssl-watchdog '133' */
 #endif
 
 #ifdef MAKE_WITH_SCHED_YIELD
-  bool disableSchedYield;       /* --disable-schedyield '134' */
+  bool disableSchedYield;        /* --disable-schedyield '134' */
 #endif
 
-  bool w3c;                     /* --w3c '136' */
+  bool w3c;                      /* --w3c '136' */
 
-  char *P3Pcp;                  /* --p3p-cp '137' */
-  char *P3Puri;                 /* --p3p-uri '138' */
+  char *P3Pcp;                   /* --p3p-cp '137' */
+  char *P3Puri;                  /* --p3p-uri '138' */
 
-  char *instance;               /* --instance '140' */
+  char *instance;                /* --instance '140' */
   char *logo;
-  bool disableStopcap;          /* --disable-stopcap '142' */
+  bool disableStopcap;           /* --disable-stopcap '142' */
 
-  bool disableInstantSessionPurge; /* --disable-instantsessionpurge '144' */
-  bool printFcOnly;             /* --fc-only '147' */
-  bool printIpOnly;             /* --no-fc '148' */
-  bool noInvalidLunDisplay;     /* --no-invalid-lun '149' */
-  bool disableMutexExtraInfo;   /* --disable-mutexextrainfo '145' */
-  bool skipVersionCheck;        /* --skip-version-check '150' */
-  char *knownSubnets;           /* --known-subnets '151' */
+  bool disableInstantSessionPurge;  /* --disable-instantsessionpurge '144' */
+  bool printFcOnly;              /* --fc-only '147' */
+  bool printIpOnly;              /* --no-fc '148' */
+  bool noInvalidLunDisplay;      /* --no-invalid-lun '149' */
+  bool disableMutexExtraInfo;    /* --disable-mutexextrainfo '145' */
+  bool skipVersionCheck;         /* --skip-version-check '150' */
+  char *knownSubnets;            /* --known-subnets '151' */
 } UserPref;
 
 typedef struct ntopGlobals {
@@ -2201,7 +2194,7 @@ typedef struct ntopGlobals {
   NtopInterface *device;   /* pointer to the network interfaces table */
 
   /* Database */
-  GDBM_FILE dnsCacheFile, pwFile, addressQueueFile, prefsFile, macPrefixFile, fingerprintFile;
+  GDBM_FILE dnsCacheFile, pwFile, prefsFile, macPrefixFile, fingerprintFile;
 
   /* the table of broadcast entries */
   HostTraffic *broadcastEntry;
@@ -2218,9 +2211,6 @@ typedef struct ntopGlobals {
   /* Multi-thread related */
   unsigned short numThreads;       /* # of running threads */
 
-  ConditionalVariable queueAddressCondvar;
-  PthreadMutex queueAddressMutex;
-
   pthread_t mainThreadId;
 
   /*
@@ -2262,7 +2252,7 @@ typedef struct ntopGlobals {
   /*
    * Control mutexes
    */
-  PthreadMutex gdbmMutex;
+  PthreadMutex gdbmMutex, portsMutex;
   PthreadMutex tcpSessionsMutex[NUM_SESSION_MUTEXES];
   PthreadMutex fcSessionsMutex; /* One mutex should be enough */
   PthreadMutex purgePortsMutex;
@@ -2304,46 +2294,6 @@ typedef struct ntopGlobals {
     dnsSniffARPACount,
     dnsSniffStoredInCache;
 
-  u_long addressQueuedCount;
-  u_int addressQueuedDup, addressQueuedCurrent, addressQueuedMax;
-
-  /*
-   *  We count calls to ipaddr2str()
-   *       {numipaddr2strCalls}
-   *    These are/are not resolved from cache.
-   *       {numFetchAddressFromCacheCalls}
-   *       {numFetchAddressFromCacheCallsOK}
-   *       {numFetchAddressFromCacheCallsFAIL}
-   *       {numFetchAddressFromCacheCallsSTALE}
-   *    Unfetched end up in resolveAddress() directly or via the queue if we have ASYNC
-   *       {numResolveAddressCalls}
-   *    In resolveAddress(), we have
-   *       {numResolveNoCacheDB} - i.e. ntop is shutting down
-   *    Otherwise we look it up (again) in the database
-   *       {numResolveCacheDBLookups}
-   *       {numResolvedFromCache} - these were basically sniffed while in queue!
-   *
-   *    Gives calls to the dns resolver:
-   *       {numResolvedFromHostAddresses} - /etc/hosts file (if we use it)
-   */
-  u_long numipaddr2strCalls,
-    numFetchAddressFromCacheCalls,
-    numFetchAddressFromCacheCallsOK,
-    numFetchAddressFromCacheCallsFAIL,
-    numFetchAddressFromCacheCallsSTALE,
-    numResolveAddressCalls,
-    numResolveNoCacheDB,
-    numResolveCacheDBLookups,
-    numResolvedFromCache,
-    dnsCacheStoredLookup,
-    numAttemptingResolutionWithDNS,
-    numResolvedWithDNSAddresses,
-    numDNSErrorHostNotFound,
-    numDNSErrorNoData,
-    numDNSErrorNoRecovery,
-    numDNSErrorTryAgain,
-    numDNSErrorOther,
-    numKeptNumericAddresses;
 #ifdef PARM_USE_HOST
   u_long  numResolvedFromHostAddresses;
 #endif
@@ -2468,14 +2418,6 @@ typedef struct ntopGlobals {
     numVendorLookupFoundMulticast,
     numVendorLookupFoundLAA;
 
-  /* Country flags */
-  IPNode *countryFlagHead;
-  int  ipCountryMem, ipCountryCount;
-
-  /* AS */
-  IPNode *asHead;
-  int    asMem, asCount;
-
   /* Memory usage */
   int piMem, ippmtmMem;
 
@@ -2499,16 +2441,18 @@ typedef struct ntopGlobals {
       queueBufferCount,
       processBufferInit,
       processBufferCount;
-  float qmaxDelay,
-        pmaxDelay;
+  float qmaxDelay, pmaxDelay;
 #endif
 
 #ifdef PARM_ENABLE_EXPERIMENTAL
   u_short experimentalFlagSet;  /* Is the 'experimental' flag set? */
 #endif
 
-  u_int8_t  calculateEfficiency;
   /* If the traffic is divided in cells (e.g. ATM, cell payload is 47 bytes) this is the cell lenght */
   u_int16_t cellLength; 
+
+  /* GeoIP */
+  GeoIP *geo_ip_db, *geo_ip_asn_db;
+
 } NtopGlobals;
 
