@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
  *
  * 			    http://www.ntop.org/
  *
@@ -312,15 +312,6 @@ void freeHostInfo(HostTraffic *host, int actualDeviceId) {
 #endif
     else
       key_data.dsize = 0;
-
-    if(key_data.dsize && (key_data.dptr != NULL)) {
-      gdbm_delete(myGlobals.addressQueueFile, key_data);
-
-#ifdef DNS_DEBUG
-      traceEvent(CONST_TRACE_INFO, "HOST_FREE_DEBUG: Deleting from GDBM address cache host [%s/%s]",
-		 host->hostNumIpAddress, host->hostResolvedName);
-#endif
-    }
   }
 
   handlePluginHostCreationDeletion(host, (u_short)actualDeviceId, 0 /* host deletion */);
@@ -472,11 +463,11 @@ void freeHostInfo(HostTraffic *host, int actualDeviceId) {
   host->dnsDomainValue = NULL;
   if(host->dnsTLDValue != NULL) free(host->dnsTLDValue);
   host->dnsTLDValue = NULL;
+  if(host->hostASDescr != NULL) free(host->hostASDescr);
   if(host->description != NULL) free(host->description);
   if(host->hwModel != NULL) free(host->hwModel);
   if(host->community != NULL) free(host->community);
-  if(host->ip2ccValue != NULL) free(host->ip2ccValue);
-  host->ip2ccValue = NULL;
+  if(host->geo_ip) GeoIPRecord_delete(host->geo_ip);
 
   /* ********** */
   /*
@@ -641,7 +632,6 @@ int purgeIdleHosts(int actDevice) {
 	  /* Host selected for deletion */
 	  theFlaggedHosts[numHosts++] = el;
 	  el->magic = CONST_UNMAGIC_NUMBER;
-	  purgeQueuedV4HostAddress(el->hostIpAddress.Ip4Address.s_addr);
 	  remove_valid_ptr(el);
 	  next = el->next;
 
