@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2003      Abdelkader Lahmadi <Abdelkader.Lahmadi@loria.fr>
  *                         Olivier Festor <Olivier.Festor@loria.fr>
- * Copyright (C) 2003-04   Luca Deri <deri@ntop.org>
+ * Copyright (C) 2003-09   Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -39,13 +39,15 @@
 /* ************************************************* */
 
 static struct in6_addr *in6_cpy(struct in6_addr *dst, struct in6_addr *src) {
-  return memcpy(dst, src, sizeof(struct in6_addr));
+  memcpy(dst, src, sizeof(struct in6_addr));
+  return(dst);
 }
 
 /* ************************************************* */
 
 static struct in_addr *in4_cpy(struct in_addr *dst, struct in_addr *src) {
-  return memcpy(dst, src, sizeof(struct in_addr));
+  memcpy(dst, src, sizeof(struct in_addr));
+  return(dst);
 }
 
 /* ************************************************* */
