@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- * Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
+ * Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -25,8 +25,6 @@
 #include "ntop.h"
 #include "globals-report.h"
 
-static void initIPCountryTable(void); /* Forward */
-
 /*
  * calculate the domain name for this host
  */
@@ -111,10 +109,31 @@ static void setDomainName(void) {
  */
 void initIPServices(void) {
   FILE* fd;
-  int idx, numSlots, len;
+  int idx, numSlots, len, rc;
+  int major, minor;
 
   traceEvent(CONST_TRACE_NOISY, "Initializing IP services");
 
+  event_init();
+
+  sscanf(event_get_version(), "%d.%d", &major, &minor);
+
+  if(minor < 4) {
+    traceEvent(CONST_TRACE_ERROR, 
+	       "You are using libevent %d whereas ntop needs at least v1.4",
+	       event_get_version());
+    traceEvent(CONST_TRACE_ERROR, 
+	       "Due to a libevent bug with IPv6 address resolution");
+traceEvent(CONST_TRACE_ERROR, 
+	   "See http://www.mail-archive.com/debian-bugs-dist@lists.debian.org/msg408382.html");
+     traceEvent(CONST_TRACE_ERROR, 
+		"Please rebuild ntop against a newer libevent version");
+   exit(0);
+  }
+
+  if((rc = evdns_init()) != DNS_ERR_NONE) {
+    traceEvent(CONST_TRACE_ERROR, "evdns_init() returned %d", rc);
+  }
 
   /* Let's count the entries first */
   numSlots = 0;
@@ -209,66 +228,6 @@ void initIPServices(void) {
 
 /* ******************************* */
 
-static void initIPCountryTable(void) {
-  u_char compressedFormat;
-  char buf[LEN_GENERAL_WORK_BUFFER];
-  FILE* fd;
-
-  myGlobals.ipCountryCount = 0;
-  if((myGlobals.countryFlagHead = malloc(sizeof(IPNode))) == NULL) {
-    traceEvent(CONST_TRACE_FATALERROR, "IP2CC: Unable to allocate table memory. Quitting...");
-    exit(4); /* Just in case */
-  }
-  myGlobals.ipCountryMem += sizeof(IPNode);
-
-  strcpy(myGlobals.countryFlagHead->node.cc, "***");
-
-  myGlobals.countryFlagHead->b[0]=NULL;
-  myGlobals.countryFlagHead->b[1]=NULL;
-
-  if (!myGlobals.runningPref.printFcOnly) {
-    fd=checkForInputFile("IP2CC",
-			 "IP address <-> Country Code mapping",
-			 CONST_P2C_FILE,
-			 NULL,
-			 &compressedFormat);
-      if(fd != NULL) {
-          char *strtokState, *cc, *ip, *prefix;
-          int numRead=0;
-
-          while(readInputFile(fd,
-                              "IP2CC",
-                              FALSE,
-                              compressedFormat,
-			      10000,
-                              buf, sizeof(buf),
-                              &numRead) == 0) {
-              if((cc=strtok_r(buf, ":", &strtokState))       == NULL) continue;
-              if((ip=strtok_r(NULL, "/", &strtokState))      == NULL) continue;
-              if((prefix=strtok_r(NULL, "\n", &strtokState)) == NULL) continue;
-
-              strtolower(cc);
-
-              if(addNodeInternal(xaton(ip), atoi(prefix), cc, 0) == NULL) {
-                traceEvent(CONST_TRACE_FATALERROR,
-                           "IP2CC: Insufficient memory to load table");
-                /* This isn't fatal of an itself, but we'll fail every malloc() from here on ... */
-                exit(5); /* Just in case */
-              }
-          }
-
-          myGlobals.ipCountryCount += numRead;
-      } else {
-          traceEvent(CONST_TRACE_WARNING,
-                     "IP2CC: Unable to read IP address <-> Country code mapping file (non-existant or no data)");
-          traceEvent(CONST_TRACE_INFO,
-                     "IP2CC: ntop will perform correctly but without this minor feature");
-      }
-  }
-}
-
-/* ******************************* */
-
 void createDeviceIpProtosList(int devIdx) {
   size_t len = (size_t)myGlobals.numIpProtosList*sizeof(TrafficCounter);
 
@@ -528,59 +487,8 @@ void initCounters(void) {
   myGlobals.sessionsCacheReused = 0;
 #endif
 
-  /*
-   * Process ASN file
-   */
-  numRead=0;
-  myGlobals.haveASN = FALSE;
-  if (!myGlobals.runningPref.printFcOnly) {
-      fd=checkForInputFile("ASN",
-                           "Autonomous System Number table",
-                           CONST_ASLIST_FILE,
-                           NULL,
-                           &compressedFormat);
-      if(fd != NULL) {
-          char *strtokState, *as, *ip, *prefix;
-
-          memset(&buf, 0, sizeof(buf));
-
-          myGlobals.asHead = malloc(sizeof(IPNode));
-          memset(myGlobals.asHead, 0, sizeof(IPNode));
-          myGlobals.asHead->node.as = 0;
-          myGlobals.asMem += sizeof(IPNode);
-
-          while(readInputFile(fd,
-                              "ASN",
-                              FALSE,
-                              compressedFormat,
-                              25000,
-                              buf, sizeof(buf),
-                              &numRead) == 0) {
-
-              if((as = strtok_r(buf, ":", &strtokState)) == NULL)  continue;
-              if((ip = strtok_r(NULL, "/", &strtokState)) == NULL)  continue;
-              if((prefix = strtok_r(NULL, "\n", &strtokState)) == NULL)  continue;
-
-              if(addNodeInternal(xaton(ip), atoi(prefix), NULL, atoi(as)) == NULL) {
-                traceEvent(CONST_TRACE_FATALERROR,
-                           "ASN: Insufficient memory to load table");
-                /* This isn't fatal of an itself, but we'll fail every malloc() from here on ... */
-                exit(6); /* Just in case */
-              }
-
-              myGlobals.asCount++;
-          }
-          traceEvent(CONST_TRACE_INFO, "ASN: ....Used %d KB of memory (%d per entry)",
-                     ((myGlobals.asMem+512)/1024), (int)sizeof(IPNode));
-          myGlobals.haveASN = TRUE;
-      } else
-          traceEvent(CONST_TRACE_NOISY, "ASN: ntop continues ok, but without ASN information.");
-  }
-
-  initIPCountryTable();
 }
 
-
 /* ******************************* */
 
 void resetStats(int deviceId) {
@@ -787,6 +695,7 @@ void reinitMutexes (void) {
  */
   createMutex(&myGlobals.logViewMutex);
   createMutex(&myGlobals.gdbmMutex);        /* data to synchronize thread access to db files */
+  createMutex(&myGlobals.portsMutex);       /* Avoid race conditions while handling ports */
 
   for(i=0; i<NUM_SESSION_MUTEXES; i++)
     createMutex(&myGlobals.tcpSessionsMutex[i]); /* data to synchronize TCP sessions access */
@@ -828,12 +737,14 @@ void initThreads(void) {
   }
   */
 
-  /*
-   * Create the thread (3) - SFP - Scan Fingerprints
-   */
-  createThread(&myGlobals.scanFingerprintsThreadId, scanFingerprintLoop, NULL);
-  traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: SFP: Started thread for fingerprinting",
-             (long)myGlobals.scanFingerprintsThreadId);
+  if(!myGlobals.runningPref.liveMode) {
+    /*
+     * Create the thread (3) - SFP - Scan Fingerprints
+     */
+    createThread(&myGlobals.scanFingerprintsThreadId, scanFingerprintLoop, NULL);
+    traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: SFP: Started thread for fingerprinting",
+	       (long)myGlobals.scanFingerprintsThreadId);
+  }
 
   /*
    * Create the thread (4) - SIH - Scan Idle Hosts - optional
@@ -842,21 +753,6 @@ void initThreads(void) {
   traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: SIH: Started thread for idle hosts detection",
              (long)myGlobals.scanIdleThreadId);
 
-  if(myGlobals.runningPref.numericFlag == 0) {
-    createMutex(&myGlobals.addressResolutionMutex);
-
-    myGlobals. numDequeueAddressThreads = MAX_NUM_DEQUEUE_ADDRESS_THREADS;
-
-    /*
-     * Create the thread (6) - DNSAR - DNS Address Resolution - optional
-     */
-    for(i=0; i<myGlobals.numDequeueAddressThreads; i++) {
-      createThread(&myGlobals.dequeueAddressThreadId[i], dequeueAddress, (char*)((long)i));
-      traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: DNSAR(%d): Started thread for DNS address resolution",
-		 (long)myGlobals.dequeueAddressThreadId[i], i+1);
-    }
-  }
-
 #ifdef MAKE_WITH_SSLWATCHDOG
 #ifdef MAKE_WITH_SSLWATCHDOG_RUNTIME
   if(myGlobals.runningPref.useSSLwatchdog == 1)
@@ -1162,32 +1058,35 @@ void addDevice(char* deviceName, char* deviceDescr) {
 		 myGlobals.device[deviceId].numHosts);
     }
 
-    memlen = sizeof(TrafficEntry*)*myGlobals.device[deviceId].numHosts*myGlobals.device[deviceId].numHosts;
-    myGlobals.device[deviceId].ipTrafficMatrix = (TrafficEntry**)calloc(myGlobals.device[deviceId].numHosts
-									*myGlobals.device[deviceId].numHosts,
-									sizeof(TrafficEntry*));
-    if(myGlobals.device[deviceId].ipTrafficMatrix == NULL) {
-      traceEvent(CONST_TRACE_FATALERROR, "Memory allocation (%d bytes) for ipTraffixMatrix failed", memlen);
-      exit(12); /* Just in case */
-    }
+    if(myGlobals.runningPref.liveMode) {
+      myGlobals.device[deviceId].ipTrafficMatrix = NULL;
+    } else {
+      memlen = sizeof(TrafficEntry*)*myGlobals.device[deviceId].numHosts*myGlobals.device[deviceId].numHosts;
+      myGlobals.device[deviceId].ipTrafficMatrix = (TrafficEntry**)calloc(myGlobals.device[deviceId].numHosts
+									  *myGlobals.device[deviceId].numHosts,
+									  sizeof(TrafficEntry*));
+      if(myGlobals.device[deviceId].ipTrafficMatrix == NULL) {
+	traceEvent(CONST_TRACE_FATALERROR, "Memory allocation (%d bytes) for ipTraffixMatrix failed", memlen);
+	exit(12); /* Just in case */
+      }
 
-    traceEvent(CONST_TRACE_NOISY, "MEMORY: ipTrafficMatrix base (no TrafficEntry) for interface '%s' is %5.2fMB",
-	       myGlobals.device[deviceId].name,
-	       ((float)(memlen)/(float)(1024.0*1024.0))+0.05);
-    myGlobals.ipTrafficMatrixMemoryUsage += memlen;
+      traceEvent(CONST_TRACE_NOISY, "MEMORY: ipTrafficMatrix base (no TrafficEntry) for interface '%s' is %5.2fMB",
+		 myGlobals.device[deviceId].name,
+		 ((float)(memlen)/(float)(1024.0*1024.0))+0.05);
+      myGlobals.ipTrafficMatrixMemoryUsage += memlen;
 
-    memlen = sizeof(struct hostTraffic*)*myGlobals.device[deviceId].numHosts;
-    myGlobals.device[deviceId].ipTrafficMatrixHosts = (struct hostTraffic**)calloc(sizeof(struct hostTraffic*),
-										   myGlobals.device[deviceId].numHosts);
+      memlen = sizeof(struct hostTraffic*)*myGlobals.device[deviceId].numHosts;
+      myGlobals.device[deviceId].ipTrafficMatrixHosts = (struct hostTraffic**)calloc(sizeof(struct hostTraffic*),
+										     myGlobals.device[deviceId].numHosts);
 
-    if(myGlobals.device[deviceId].ipTrafficMatrixHosts == NULL) {
-      traceEvent(CONST_TRACE_FATALERROR, "Memory allocation (%d bytes) for ipTraffixMatrixHosts failed", memlen);
-      exit(13); /* Just in case */
-    }
+      if(myGlobals.device[deviceId].ipTrafficMatrixHosts == NULL) {
+	traceEvent(CONST_TRACE_FATALERROR, "Memory allocation (%d bytes) for ipTraffixMatrixHosts failed", memlen);
+	exit(13); /* Just in case */
+      }
 
-    /* Allocate FC Traffic Matrices */
+      /* Allocate FC Traffic Matrices */
 #ifdef NOT_YET
-    if (!myGlobals.runningPref.printIpOnly) {
+      if (!myGlobals.runningPref.printIpOnly) {
         memlen = sizeof(TrafficEntry*)*myGlobals.device[deviceId].numHosts*myGlobals.device[deviceId].numHosts;
         myGlobals.device[deviceId].fcTrafficMatrix = (TrafficEntry**)calloc(myGlobals.device[deviceId].numHosts
                                                                             *myGlobals.device[deviceId].numHosts,
@@ -1201,8 +1100,9 @@ void addDevice(char* deviceName, char* deviceDescr) {
         memlen = sizeof(struct hostTraffic*)*myGlobals.device[deviceId].numHosts;
         myGlobals.device[deviceId].fcTrafficMatrixHosts = (struct hostTraffic**)calloc(sizeof(struct hostTraffic*),
                                                                                        myGlobals.device[deviceId].numHosts);
-    }
+      }
 #endif
+    }
   }
 
   /* ********************************************* */
@@ -1221,35 +1121,26 @@ void addDevice(char* deviceName, char* deviceDescr) {
     if(myGlobals.device[i].name != NULL)
       mallocLen += strlen(myGlobals.device[i].name) + 2;
   }
-  workDevices = malloc(mallocLen);
+
+  workDevices = calloc(mallocLen+1, 1);
   if(workDevices == NULL)
     return;
   else
-    memset(workDevices, 0, mallocLen);
 
   for(i=0; i<myGlobals.numDevices; i++) {
     if(myGlobals.device[i].name != NULL) {
-      int hasSpace;
-
-      if(i>0)
-        strncat(workDevices, ", ", (sizeof(workDevices) - strlen(workDevices) - 1));
-
-      if(myGlobals.device[i].name[strlen(myGlobals.device[i].name)-1] == ' ')
-	hasSpace = 1;
-      else
-	hasSpace = 0;
-
-      strncat(workDevices,
-              myGlobals.device[i].name,
-              (sizeof(workDevices) - strlen(workDevices) - hasSpace));
+      int len = strlen(workDevices);
+      safe_snprintf(__FILE__, __LINE__, 
+		    &workDevices[len], mallocLen-len, 
+		    "%s%s", (i > 0) ? ", " : "",
+		    myGlobals.device[i].name);
     }
   }
 
   if(myGlobals.runningPref.devices != NULL)
     free(myGlobals.runningPref.devices);
 
-  myGlobals.runningPref.devices = strdup(workDevices);
-  free(workDevices);
+  myGlobals.runningPref.devices = workDevices;
 
   /* ********************************************** */
 
@@ -1606,24 +1497,8 @@ void parseTrafficFilter(void) {
     int i;
     struct bpf_program fcode;
 
-    for(i=0; i<myGlobals.numDevices; i++) {
-      if(myGlobals.device[i].pcapPtr && (!myGlobals.device[i].virtualDevice)) {
-	if((pcap_compile(myGlobals.device[i].pcapPtr, &fcode, myGlobals.runningPref.currentFilterExpression, 1,
-			 myGlobals.device[i].netmask.s_addr) < 0)
-	   || (pcap_setfilter(myGlobals.device[i].pcapPtr, &fcode) < 0)) {
-	  traceEvent(CONST_TRACE_FATALERROR,
-		     "Wrong filter '%s' (%s) on interface %s",
-		     myGlobals.runningPref.currentFilterExpression,
-		     pcap_geterr(myGlobals.device[i].pcapPtr),
-		     myGlobals.device[i].name[0] == '0' ? "<pcap file>" : myGlobals.device[i].name);
-	  exit(15); /* Just in case */
-	}
-
-        traceEvent(CONST_TRACE_NOISY, "Setting filter to \"%s\" on device %s.",
-		     myGlobals.runningPref.currentFilterExpression, myGlobals.device[i].name);
-        pcap_freecode(&fcode);
-      }
-    }
+    for(i=0; i<myGlobals.numDevices; i++)
+      setPcapFilter(myGlobals.runningPref.currentFilterExpression, i);
   } else
     myGlobals.runningPref.currentFilterExpression = strdup("");	/* so that it isn't NULL! */
 }
@@ -1698,6 +1573,7 @@ u_int createDummyInterface(char *ifName) {
   u_int mallocLen;
 #endif
   u_int deviceId = myGlobals.numDevices;
+  int i;
 
   traceEvent(CONST_TRACE_INFO, "Creating dummy interface, '%s'", ifName);
 
@@ -1721,7 +1597,7 @@ u_int createDummyInterface(char *ifName) {
   myGlobals.device[deviceId].virtualDevice = 0;
   myGlobals.device[deviceId].activeDevice  = 0;
   myGlobals.device[deviceId].samplingRate =  myGlobals.runningPref.samplingRate;
-  calculateUniqueInterfaceName(deviceId);
+  calculateUniqueInterfaceName(deviceId); 
 
   if(myGlobals.otherHostEntry != NULL) {
     myGlobals.device[deviceId].hash_hostTraffic[OTHER_HOSTS_ENTRY] = myGlobals.otherHostEntry;
@@ -1754,5 +1630,14 @@ u_int createDummyInterface(char *ifName) {
     memset(myGlobals.device[deviceId].tcpSession, 0, len);
   }
 
+  /* Allocate memory for dhcp stats */
+
+  for(i=0; i<myGlobals.numKnownSubnets; i++) {
+    myGlobals.device[deviceId].networkHost[i].protocolInfo = calloc(1, sizeof(ProtocolInfo));
+    myGlobals.device[deviceId].networkHost[i].protocolInfo->dnsStats = calloc(1, sizeof(ServiceStats));
+    myGlobals.device[deviceId].networkHost[i].protocolInfo->httpStats = calloc(1, sizeof(ServiceStats));
+    myGlobals.device[deviceId].networkHost[i].protocolInfo->dhcpStats = calloc(1, sizeof(DHCPStats));
+  }
+
   return(deviceId);
 }
