@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- * Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
+ *         Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -48,16 +48,10 @@
 #ifdef MAKE_WITH_SAFER_ROUTINES
 
 static void stopcap(void) {
-  int i;
-
   traceEvent(CONST_TRACE_WARNING, "ntop packet capture STOPPED");
   traceEvent(CONST_TRACE_INFO, "NOTE: ntop web server remains up");
   traceEvent(CONST_TRACE_INFO, "NOTE: Shutdown gracefully and restart with more memory");
   setRunState(FLAG_NTOPSTATE_STOPCAP);
-
-  for(i=0; i<myGlobals.numDevices; i++) signalCondvar(&myGlobals.device[i].queueCondvar);
-
-  signalCondvar(&myGlobals.queueAddressCondvar);
 }
 
 #undef malloc /* just to be safe */
