@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- * Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
+ *            Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -171,9 +171,7 @@ RETSIGTYPE handleDiedChild(int sig _UNUSED_) {
 
 /* **************************************** */
 
-
 void daemonizeUnderUnix(void) {
-
 #ifndef WIN32
   int childpid;
 
@@ -249,7 +247,6 @@ void detachFromTerminalUnderUnix(int doChdir) {
 #endif
 
 #endif /* WIN32 */
-
 }
 
 /* **************************************** */
@@ -448,7 +445,8 @@ void createPortHash(void) {
 /* **************************************** */
 
 void handleProtocols(void) {
-  char *proto, *buffer=NULL, *strtokState, *bufferCurrent, *bufferWork;
+  char *proto, *buffer=NULL, *strtokState,
+	   *bufferCurrent, *bufferWork, tmpStr[512];
   FILE *fd;
 
   /* myGlobals.protoSpecs is either
@@ -463,25 +461,29 @@ void handleProtocols(void) {
      || (!myGlobals.runningPref.protoSpecs[0]))
     return;
 
-  fd = fopen(myGlobals.runningPref.protoSpecs, "rb");
+   safe_snprintf(__FILE__, __LINE__, tmpStr, sizeof(tmpStr),
+                  "%s", myGlobals.runningPref.protoSpecs);
+
+  revertSlashIfWIN32(tmpStr, 0);
+  fd = fopen(tmpStr, "rb");
 
   if(fd == NULL) {
-    traceEvent(CONST_TRACE_INFO, "PROTO_INIT: Processing protocol list: '%s'", myGlobals.runningPref.protoSpecs);
-    proto = strtok_r(myGlobals.runningPref.protoSpecs, ",", &strtokState);
+    traceEvent(CONST_TRACE_INFO, "PROTO_INIT: Processing protocol list: '%s'", tmpStr);
+    proto = strtok_r(tmpStr, ",", &strtokState);
   } else {
     struct stat buf;
 
-    if(stat(myGlobals.runningPref.protoSpecs, &buf) != 0) {
+    if(stat(tmpStr, &buf) != 0) {
       fclose(fd);
       traceEvent(CONST_TRACE_ERROR, "PROTO_INIT: Unable to get information about file '%s'",
-		 myGlobals.runningPref.protoSpecs);
+		 tmpStr);
       return;
     }
 
     bufferCurrent = buffer = (char*)malloc(buf.st_size+8) /* just to be safe */;
 
     traceEvent(CONST_TRACE_ALWAYSDISPLAY, "PROTO_INIT: Processing protocol file: '%s', size: %ld",
-	       myGlobals.runningPref.protoSpecs, (long)(buf.st_size+8));
+	       tmpStr, (long)(buf.st_size+8));
 
     for (;;) {
       bufferCurrent = fgets(bufferCurrent, buf.st_size, fd);
@@ -512,6 +514,11 @@ void handleProtocols(void) {
 	bufferWork[1] = '\0';
       }
 
+    bufferWork = strchr(bufferCurrent, '\r');
+      if(bufferWork != NULL) {
+	bufferWork[0] = ',';
+	bufferWork[1] = '\0';
+      }
       /* Move pointer to end-of-string for read of next line */
       bufferCurrent = strchr(bufferCurrent, '\0');
     }
@@ -546,8 +553,6 @@ void handleProtocols(void) {
 		   "PROTO_INIT: Unknown protocol '%s'. It has been ignored",
 		   proto);
       else {
-	char tmpStr[255];
-
 	protoName[0] = '\0';
 	memset(tmpStr, 0, sizeof(tmpStr));
 	strncpy(tmpStr, &protoName[1], sizeof(tmpStr));
@@ -785,13 +790,6 @@ static void cleanupThreadIs(char *buf, int sizeofbuf) {
     else if(pthread_self() == myGlobals.sslwatchdogChildThreadId)
       strncpy(buf, "SSL", sizeofbuf);
 #endif
-    else
-      for(i=0; i<myGlobals.numDequeueAddressThreads; i++) {
-	if(pthread_self() == myGlobals.dequeueAddressThreadId[i]) {
-	  safe_snprintf(__FILE__, __LINE__, buf, sizeofbuf, "DNSAR%d", i+1);
-	  break;
-	}
-      }
   }
 
   if(buf[0] == '\0') {
@@ -864,21 +862,6 @@ void runningThreads(char *buf, int sizeofbuf, int do_join) {
 
   }
 
-  for(i=0; i<myGlobals.numDequeueAddressThreads; i++) {
-    if(myGlobals.dequeueAddressThreadId[i] != 0) {
-      if(!do_join) {
-	safe_snprintf(__FILE__, __LINE__, buf2, sizeof(buf2), " DNSAR%d", i+1);
-	safe_strncat(buf, sizeofbuf, buf2);
-      } else {
-	/* Wake up thread */
-	signalCondvar(&myGlobals.queueAddressCondvar);
-	traceEvent(CONST_TRACE_INFO, "Joining thread DNSAR%d", i+1);
-	if(joinThread(&myGlobals.dequeueAddressThreadId[i]) != 0)
-	  traceEvent(CONST_TRACE_INFO, "joinThread() returned %s", strerror(errno));
-      }
-    }
-  }
-
   if(myGlobals.allDevs != NULL) {
     pcap_freealldevs(myGlobals.allDevs);
     myGlobals.allDevs = NULL;
@@ -911,8 +894,9 @@ void runningThreads(char *buf, int sizeofbuf, int do_join) {
 	  /* signalCondvar(&myGlobals.device[i].queueCondvar); */
 	  pcap_close(myGlobals.device[i].pcapPtr);
 
-	  traceEvent(CONST_TRACE_INFO, "Joining thread  NPS(%s)",
-		     myGlobals.device[i].humanFriendlyName);
+	  traceEvent(CONST_TRACE_INFO, "Joining thread NPS(%s) [t%lu]",
+		     myGlobals.device[i].humanFriendlyName,
+		     (long)myGlobals.device[i].pcapDispatchThreadId);
 	  if(joinThread(&myGlobals.device[i].pcapDispatchThreadId) != 0)
 	    traceEvent(CONST_TRACE_INFO, "joinThread() returned: %s", strerror(errno));
 	}
@@ -1129,11 +1113,18 @@ RETSIGTYPE cleanup(int signo) {
     deleteMutex(&myGlobals.hostsHashMutex[i]);
   }
 
-  deleteCondvar(&myGlobals.queueAddressCondvar);
-  deleteMutex(&myGlobals.queueAddressMutex);
-
   termGdbm();
   termDB();
+  
+  if(myGlobals.geo_ip_db != NULL) {
+    GeoIP_delete(myGlobals.geo_ip_db);
+    myGlobals.geo_ip_db = NULL;
+  }
+
+  if(myGlobals.geo_ip_asn_db != NULL) {
+    GeoIP_delete(myGlobals.geo_ip_asn_db);
+    myGlobals.geo_ip_asn_db = NULL;
+  }
 
   tryLockMutex(&myGlobals.gdbmMutex, "cleanup");
   deleteMutex(&myGlobals.gdbmMutex);
