@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
  *
  *  			    http://www.ntop.org/
  *
@@ -456,7 +456,6 @@ void printAvailableInterfaces() {
 
 #define	MAX_WIN32_NET_ALIASES 35
 
-static char NETDB[] = CONST_WIN32_PATH_NETWORKS;
 static FILE *netf = NULL;
 static char line[BUFSIZ+1];
 static struct netent net;
@@ -467,27 +466,6 @@ int _net_stayopen;
 
 /* ************************************ */
 
-void setnetent(int f)
-{
-  if(netf == NULL)
-    netf = fopen(NETDB, "r" );
-  else
-    rewind(netf);
-  _net_stayopen |= f;
-}
-
-/* ************************************ */
-
-void endnetent() {
-  if(netf) {
-    fclose(netf);
-    netf = NULL;
-  }
-  _net_stayopen = 0;
-}
-
-/* ************************************ */
-
 static char *any(char *cp, char *match) {
   register char *mp, c;
 
@@ -559,6 +537,8 @@ u_int32_t inet_network(const char *cp) {
 
 /* ************************************ */
 
+#if 0
+
 struct netent* getnetent() {
   char *p;
   register char *cp, **q;
@@ -624,6 +604,7 @@ struct netent *getnetbyname(const char *name) {
     endnetent();
   return (p);
 }
+#endif
 
 /* ************************************ */
 
