@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2008 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
  *                      
  *  			    http://www.ntop.org/
  *  					
@@ -68,6 +68,7 @@ typedef int NDIS_STATUS, *PNDIS_STATUS;
 #define strcasecmp _stricmp
 #define snprintf _snprintf
 
+#define HAVE_GETOPT_LONG
 
 extern int getopt(int num, char *const *argv, const char *opts);
 #define getopt getopt____
@@ -282,16 +283,19 @@ struct icmp_ra_addr
   u_int32_t ira_preference;
 };
 
+/*                                                                                                                                                                        
+ * Structure of an icmp header.                                                                                                                                           
+ */
 struct icmp
 {
-  u_int8_t  icmp_type;	/* type of message, see below */
-  u_int8_t  icmp_code;	/* type sub code */
-  u_int16_t icmp_cksum;	/* ones complement checksum of struct */
+  u_int8_t  icmp_type;  /* type of message, see below */
+  u_int8_t  icmp_code;  /* type sub code */
+  u_int16_t icmp_cksum; /* ones complement checksum of struct */
   union
   {
-    u_char ih_pptr;		/* ICMP_PARAMPROB */
-    struct in_addr ih_gwaddr;	/* gateway address */
-    struct ih_idseq		/* echo datagram */
+    u_char ih_pptr;             /* ICMP_PARAMPROB */
+    struct in_addr ih_gwaddr;   /* gateway address */
+    struct ih_idseq             /* echo datagram */
     {
       u_int16_t icd_id;
       u_int16_t icd_seq;
@@ -312,16 +316,16 @@ struct icmp
       u_int16_t irt_lifetime;
     } ih_rtradv;
   } icmp_hun;
-#define	icmp_pptr	icmp_hun.ih_pptr
-#define	icmp_gwaddr	icmp_hun.ih_gwaddr
-#define	icmp_id		icmp_hun.ih_idseq.icd_id
-#define	icmp_seq	icmp_hun.ih_idseq.icd_seq
-#define	icmp_void	icmp_hun.ih_void
-#define	icmp_pmvoid	icmp_hun.ih_pmtu.ipm_void
-#define	icmp_nextmtu	icmp_hun.ih_pmtu.ipm_nextmtu
-#define	icmp_num_addrs	icmp_hun.ih_rtradv.irt_num_addrs
-#define	icmp_wpa	icmp_hun.ih_rtradv.irt_wpa
-#define	icmp_lifetime	icmp_hun.ih_rtradv.irt_lifetime
+#define icmp_pptr       icmp_hun.ih_pptr
+#define icmp_gwaddr     icmp_hun.ih_gwaddr
+#define icmp_id         icmp_hun.ih_idseq.icd_id
+#define icmp_seq        icmp_hun.ih_idseq.icd_seq
+#define icmp_void       icmp_hun.ih_void
+#define icmp_pmvoid     icmp_hun.ih_pmtu.ipm_void
+#define icmp_nextmtu    icmp_hun.ih_pmtu.ipm_nextmtu
+#define icmp_num_addrs  icmp_hun.ih_rtradv.irt_num_addrs
+#define icmp_wpa        icmp_hun.ih_rtradv.irt_wpa
+#define icmp_lifetime   icmp_hun.ih_rtradv.irt_lifetime
   union
   {
     struct
@@ -339,15 +343,18 @@ struct icmp
     u_int32_t   id_mask;
     u_int8_t    id_data[1];
   } icmp_dun;
-#define	icmp_otime	icmp_dun.id_ts.its_otime
-#define	icmp_rtime	icmp_dun.id_ts.its_rtime
-#define	icmp_ttime	icmp_dun.id_ts.its_ttime
-#define	icmp_ip		icmp_dun.id_ip.idi_ip
-#define	icmp_radv	icmp_dun.id_radv
-#define	icmp_mask	icmp_dun.id_mask
-#define	icmp_data	icmp_dun.id_data
+#define icmp_otime      icmp_dun.id_ts.its_otime
+#define icmp_rtime      icmp_dun.id_ts.its_rtime
+#define icmp_ttime      icmp_dun.id_ts.its_ttime
+#define icmp_ip         icmp_dun.id_ip.idi_ip
+#define icmp_radv       icmp_dun.id_radv
+#define icmp_mask       icmp_dun.id_mask
+#define icmp_data       icmp_dun.id_data
 };
 
+
+
+
 /*
  * Definition of type and code field values.
  */
