@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
  *
  *  			    http://www.ntop.org/
  *
@@ -22,7 +22,7 @@
 
 #ifdef MAKE_STATIC_PLUGIN
 extern PluginInfo* icmpPluginEntryFctn(void);
-extern PluginInfo* sflowPluginEntryFctn(void)
+extern PluginInfo* sflowPluginEntryFctn(void);
 extern PluginInfo* rrdPluginEntryFctn(void);
 extern PluginInfo* cpacketPluginEntryFctn(void);
 extern PluginInfo* netflowPluginEntryFctn(void);
