@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- * Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
+ * Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -73,6 +73,7 @@ static struct option const long_options[] = {
 #ifndef WIN32
   { "daemon",                           no_argument,       NULL, 'd' },
 #endif
+
   { "max-table-rows",                   required_argument, NULL, 'e' },
   { "traffic-dump-file",                required_argument, NULL, 'f' },
   { "track-local-hosts",                no_argument,       NULL, 'g' },
@@ -84,7 +85,6 @@ static struct option const long_options[] = {
   { "numeric-ip-addresses",             no_argument,       NULL, 'n' },
   { "no-mac",                           no_argument,       NULL, 'o' },
 
-
   { "protocols",                        required_argument, NULL, 'p' },
   { "create-suspicious-packets",        no_argument,       NULL, 'q' },
   { "refresh-time",                     required_argument, NULL, 'r' },
@@ -153,11 +153,15 @@ static struct option const long_options[] = {
   { "disable-mutexextrainfo",           no_argument,       NULL, 145 },
 
   { "fc-only",                          no_argument,       NULL, 147 },
-  { "no-fc",                            no_argument,       0, 148    },
-  { "no-invalid-lun",                   no_argument,       0, 149    },
+  { "no-fc",                            no_argument,       0,    148 },
+  { "no-invalid-lun",                   no_argument,       0,    149 },
 
   { "skip-version-check",               required_argument, NULL, 150 },
-  { "known-subnets",                    required_argument, NULL, 151 },
+  { "pcap-file-list",                   required_argument, NULL, 151 },
+  { "known-subnets",                    required_argument, NULL, 152 },
+  { "live",                             no_argument,       NULL, 153 },
+  { "enable-efficiency",                no_argument,       NULL, 154 },
+
   {NULL, 0, NULL, 0}
 };
 
@@ -307,6 +311,10 @@ static void readPcapFileList(char * filenames) {
 }
 
 /* ***************************************************** */
+/*	NOTE
+	In the function below do NOT free myGlobals.runningPref.*
+	as this value is used in myGlobals.savedPref.*
+*/
 
 /*
  * Parse the command line options
@@ -382,7 +390,6 @@ int parseOptions(int argc, char* argv[]) {
 #ifndef WIN32
       stringSanityCheck(optarg, "-i | --interface");
 #endif
-      if(myGlobals.runningPref.devices != NULL) free(myGlobals.runningPref.devices);
       myGlobals.runningPref.devices = strdup(optarg);
       break;
 
@@ -439,7 +446,6 @@ int parseOptions(int argc, char* argv[]) {
 #ifndef WIN32
     case 'u':
       stringSanityCheck(optarg, "-u | --user");
-      if(myGlobals.effectiveUserName != NULL) free(myGlobals.effectiveUserName);
       myGlobals.effectiveUserName = strdup(optarg);
       if(strOnlyDigits(optarg))
 	myGlobals.userId = atoi(optarg);
@@ -490,7 +496,8 @@ int parseOptions(int argc, char* argv[]) {
 
     case 'B':
       stringSanityCheck(optarg, "-B | --filter-expression");
-      myGlobals.runningPref.currentFilterExpression = strdup(optarg);
+      if(myGlobals.runningPref.currentFilterExpression == NULL)
+	myGlobals.runningPref.currentFilterExpression = strdup(optarg);
       break;
 
     case 'C': /* Sampling rate */
@@ -527,22 +534,17 @@ int parseOptions(int argc, char* argv[]) {
 
     case 'N':
       pathSanityCheck(optarg, "-N | --wwn-map");
-      if(myGlobals.runningPref.fcNSCacheFile != NULL)
-	   free(myGlobals.runningPref.fcNSCacheFile);
       myGlobals.runningPref.fcNSCacheFile = strdup (optarg);
       break;
 
     case 'O': /* pcap log path - Ola Lundqvist <opal@debian.org> */
       pathSanityCheck(optarg, "-O | --output-packet-path");
-      if(myGlobals.runningPref.pcapLogBasePath != NULL)
-	free(myGlobals.runningPref.pcapLogBasePath);
-      myGlobals.runningPref.pcapLogBasePath = strdup(optarg);
+       myGlobals.runningPref.pcapLogBasePath = strdup(optarg);
       break;
 
     case 'P':
       pathSanityCheck(optarg, "-P | --db-file-path");
-      if(myGlobals.dbPath != NULL)
-	free(myGlobals.dbPath);
+      if(myGlobals.dbPath != NULL) free(myGlobals.dbPath);
 
       if(optarg[strlen(optarg)-1] == '/') optarg[strlen(optarg)-1] = '\0';
       mkdir_p("dbPath", optarg, 0777);
@@ -551,8 +553,7 @@ int parseOptions(int argc, char* argv[]) {
 
     case 'Q': /* Spool Path (ntop's spool directory) */
       pathSanityCheck(optarg, "-Q | --spool-file-path" );
-      if(myGlobals.spoolPath != NULL)
-	free(myGlobals.spoolPath);
+      if(myGlobals.spoolPath != NULL) free(myGlobals.spoolPath);
       if(optarg[strlen(optarg)-1] == '/') optarg[strlen(optarg)-1] = '\0';
       mkdir_p("spoolPath", optarg, 0777);
       myGlobals.spoolPath = strdup(optarg);
@@ -671,15 +672,11 @@ int parseOptions(int argc, char* argv[]) {
 
     case 137:
       stringSanityCheck(optarg, "--p3pcp");
-      if(myGlobals.runningPref.P3Pcp != NULL)
-	free(myGlobals.runningPref.P3Pcp);
       myGlobals.runningPref.P3Pcp = strdup(optarg);
       break;
 
     case 138:
       uriSanityCheck(optarg, "--p3puri", FALSE);
-      if(myGlobals.runningPref.P3Puri != NULL)
-	free(myGlobals.runningPref.P3Puri);
       myGlobals.runningPref.P3Puri = strdup(optarg);
       break;
 
@@ -691,8 +688,6 @@ int parseOptions(int argc, char* argv[]) {
       FILE *fd;
 
       stringSanityCheck(optarg, "--instance");
-      if(myGlobals.runningPref.instance != NULL)
-        free(myGlobals.runningPref.instance);
       myGlobals.runningPref.instance = strdup(optarg);
 
       memset(&tmpStr, 0, sizeof(tmpStr));
@@ -760,6 +755,14 @@ int parseOptions(int argc, char* argv[]) {
       myGlobals.runningPref.knownSubnets = strdup(optarg);
       break;
 
+    case 153:
+      myGlobals.runningPref.liveMode = 1;
+      break;
+
+    case 154:
+      myGlobals.runningPref.calculateEfficiency = 1;
+      break;
+
     default:
       printf("FATAL ERROR: unknown ntop option, '%c'\n", opt);
 #ifdef DEBUG
@@ -771,6 +774,12 @@ int parseOptions(int argc, char* argv[]) {
     }
   }
 
+  if(myGlobals.runningPref.liveMode) {
+    myGlobals.runningPref.trackOnlyLocalHosts   = 1;
+    myGlobals.runningPref.dontTrustMACaddr      = 1;
+    myGlobals.runningPref.enableSessionHandling = 1;
+  }
+
   /* *********************** */
 
   if(setAdminPw) {
@@ -840,6 +849,8 @@ int parseOptions(int argc, char* argv[]) {
   }
 #endif
 
+  
+
   return(userSpecified);
 }
 
@@ -1002,10 +1013,10 @@ void processStrPref(char *key, char *value, char **globalVar, bool savePref)
     if (*globalVar != NULL) {
       free (*globalVar);
       *globalVar = NULL;
-      if (savePref) {
-	delPrefsValue (key);
-      }
     }
+	
+	 *globalVar = strdup(value);
+     if (savePref) delPrefsValue (key);
   }
   else {
     if (savePref) {
